"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handlePatch = handlePatch;
exports.isPatch = isPatch;
var _ramda = require("ramda");
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function isPatch(obj) {
  return (0, _ramda.has)('__dash_patch_update', obj);
}
function getLocationIndex(value, previous) {
  if ((0, _ramda.is)(Number, value) && value < 0) {
    return previous.length + value;
  }
  return value;
}
function getLocationPath(location, obj) {
  var current = [];
  for (var i = 0; i < location.length; i++) {
    var value = getLocationIndex(location[i], (0, _ramda.path)(current, obj));
    current.push(value);
  }
  return current;
}
var patchHandlers = {
  Assign: function Assign(previous, patchOperation) {
    var params = patchOperation.params,
      location = patchOperation.location;
    return (0, _ramda.assocPath)(location, params.value, previous);
  },
  Merge: function Merge(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, _objectSpread(_objectSpread({}, prev), patchOperation.params.value), previous);
  },
  Extend: function Extend(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, (0, _ramda.concat)(prev, patchOperation.params.value), previous);
  },
  Delete: function Delete(previous, patchOperation) {
    return (0, _ramda.dissocPath)(patchOperation.location, previous);
  },
  Insert: function Insert(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, (0, _ramda.insert)(getLocationIndex(patchOperation.params.index, prev), patchOperation.params.value, prev), previous);
  },
  Append: function Append(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, (0, _ramda.append)(patchOperation.params.value, prev), previous);
  },
  Prepend: function Prepend(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, (0, _ramda.prepend)(patchOperation.params.value, prev), previous);
  },
  Add: function Add(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, prev + patchOperation.params.value, previous);
  },
  Sub: function Sub(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, prev - patchOperation.params.value, previous);
  },
  Mul: function Mul(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, prev * patchOperation.params.value, previous);
  },
  Div: function Div(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, prev / patchOperation.params.value, previous);
  },
  Clear: function Clear(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, (0, _ramda.empty)(prev), previous);
  },
  Reverse: function Reverse(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, (0, _ramda.reverse)(prev), previous);
  },
  Remove: function Remove(previous, patchOperation) {
    var prev = (0, _ramda.path)(patchOperation.location, previous);
    return (0, _ramda.assocPath)(patchOperation.location, prev.filter(function (item) {
      return !(0, _ramda.equals)(item, patchOperation.params.value);
    }), previous);
  }
};
function handlePatch(previousValue, patchValue) {
  var reducedValue = previousValue;
  for (var i = 0; i < patchValue.operations.length; i++) {
    var patch = patchValue.operations[i];
    patch.location = getLocationPath(patch.location, reducedValue);
    var handler = patchHandlers[patch.operation];
    if (!handler) {
      throw new Error("Invalid Operation ".concat(patch.operation));
    }
    reducedValue = handler(reducedValue, patch);
  }
  return reducedValue;
}